package com.andengine.entity.shape;

import static org.anddev.andengine.util.constants.Constants.VERTEX_INDEX_X;
import static org.anddev.andengine.util.constants.Constants.VERTEX_INDEX_Y;

import org.anddev.andengine.collision.BaseCollisionChecker;
import org.anddev.andengine.collision.ShapeCollisionChecker;
import org.anddev.andengine.util.MathUtils;

/**
 * @author Jon Rose
 * @ code reused from AndEngine examples
 * @since 11:50:19 - 11.03.2010
 */
public class EllipticalShapeCollisionChecker extends ShapeCollisionChecker {
	// ===========================================================
	// Constants
	// ===========================================================

	private static final int ELLIPTICALSHAPE_VERTEX_COUNT = 4;

	private static final float[] VERTICES_CONTAINS_TMP = new float[2 * ELLIPTICALSHAPE_VERTEX_COUNT];
	private static final float[] VERTICES_COLLISION_TMP_A = new float[2 * ELLIPTICALSHAPE_VERTEX_COUNT];
	private static final float[] VERTICES_COLLISION_TMP_B = new float[2 * ELLIPTICALSHAPE_VERTEX_COUNT];

	// ===========================================================
	// Fields
	// ===========================================================

	// ===========================================================
	// Constructors
	// ===========================================================

	// ===========================================================
	// Getter & Setter
	// ===========================================================

	// ===========================================================
	// Methods for/from SuperClass/Interfaces
	// ===========================================================

	// ===========================================================
	// Methods
	// ===========================================================

	public static boolean checkContains(final EllipticalShape pEllipticalShape, final float pX, final float pY) {
		EllipticalShapeCollisionChecker.fillVertices(pEllipticalShape, VERTICES_CONTAINS_TMP);
		return ShapeCollisionChecker.checkContains(VERTICES_CONTAINS_TMP, 2 * ELLIPTICALSHAPE_VERTEX_COUNT, pX, pY);
	}

	public static boolean checkCollision(final EllipticalShape pEllipticalShapeA, final EllipticalShape pEllipticalShapeB) {
		if(pEllipticalShapeA.getRotation() == 0 && pEllipticalShapeB.getRotation() == 0 && pEllipticalShapeA.isScaled() == false && pEllipticalShapeB.isScaled() == false) {
			final float aLeft = pEllipticalShapeA.getX();
			final float aTop = pEllipticalShapeA.getY();
			final float bLeft = pEllipticalShapeB.getX();
			final float bTop = pEllipticalShapeB.getY();
			return BaseCollisionChecker.checkAxisAlignedRectangleCollision(aLeft, aTop, aLeft + pEllipticalShapeA.getWidth(), aTop + pEllipticalShapeA.getHeight(),
																			bLeft, bTop, bLeft + pEllipticalShapeB.getWidth(), bTop + pEllipticalShapeB.getHeight());
		} else {
			EllipticalShapeCollisionChecker.fillVertices(pEllipticalShapeA, VERTICES_COLLISION_TMP_A);
			EllipticalShapeCollisionChecker.fillVertices(pEllipticalShapeB, VERTICES_COLLISION_TMP_B);

			return ShapeCollisionChecker.checkCollision(2 * ELLIPTICALSHAPE_VERTEX_COUNT, 2 * ELLIPTICALSHAPE_VERTEX_COUNT, VERTICES_COLLISION_TMP_A, VERTICES_COLLISION_TMP_B);
		}
	}

	public static void fillVertices(final EllipticalShape pEllipticalShape, final float[] pVertices) {
		final float left = pEllipticalShape.getX();
		final float top = pEllipticalShape.getY();
		final float right = pEllipticalShape.getWidth() + left;
		final float bottom = pEllipticalShape.getHeight() + top;

		pVertices[0 + VERTEX_INDEX_X] = left;
		pVertices[0 + VERTEX_INDEX_Y] = top;

		pVertices[2 + VERTEX_INDEX_X] = right;
		pVertices[2 + VERTEX_INDEX_Y] = top;

		pVertices[4 + VERTEX_INDEX_X] = right;
		pVertices[4 + VERTEX_INDEX_Y] = bottom;

		pVertices[6 + VERTEX_INDEX_X] = left;
		pVertices[6 + VERTEX_INDEX_Y] = bottom;

		MathUtils.rotateAndScaleAroundCenter(pVertices,
				pEllipticalShape.getRotation(), left + pEllipticalShape.getRotationCenterX(), top + pEllipticalShape.getRotationCenterY(),
				pEllipticalShape.getScaleX(), pEllipticalShape.getScaleY(), left + pEllipticalShape.getScaleCenterX(), top + pEllipticalShape.getScaleCenterY());
	}
	
	public static boolean checkAxisAlignedEllipseCollision(final float pLeftA, final float pTopA, final float pRightA, final float pBottomA, final float pLeftB, final float pTopB, final float pRightB, final float pBottomB) {
		return (pLeftA < pRightB &&
				pLeftB < pRightA &&
				pTopA < pBottomB &&
				pTopB < pBottomA);
	}

	// ===========================================================
	// Inner and Anonymous Classes
	// ===========================================================
}
