package com.ghostmachine.android.twotouchxy;

import javax.microedition.khronos.opengles.GL10;

import org.anddev.andengine.engine.Engine;
import org.anddev.andengine.entity.particle.ParticleSystem;
import org.anddev.andengine.entity.particle.emitter.BaseParticleEmitter;
import org.anddev.andengine.entity.particle.emitter.PointParticleEmitter;
import org.anddev.andengine.entity.particle.modifier.AccelerationInitializer;
import org.anddev.andengine.entity.particle.modifier.AlphaModifier;
import org.anddev.andengine.entity.particle.modifier.ColorModifier;
import org.anddev.andengine.entity.particle.modifier.ExpireModifier;
import org.anddev.andengine.entity.particle.modifier.ScaleModifier;
import org.anddev.andengine.entity.particle.modifier.VelocityInitializer;
import org.anddev.andengine.entity.primitive.Line;
import org.anddev.andengine.entity.scene.Scene;
import org.anddev.andengine.entity.sprite.Sprite;
import org.anddev.andengine.entity.text.ChangeableText;
import org.anddev.andengine.input.touch.TouchEvent;
import org.anddev.andengine.opengl.font.Font;
import org.anddev.andengine.opengl.texture.Texture;
import org.anddev.andengine.opengl.texture.TextureOptions;
import org.anddev.andengine.opengl.texture.region.TextureRegion;
import org.anddev.andengine.opengl.texture.region.TextureRegionFactory;

import android.content.Context;
import android.graphics.Color;
import android.graphics.Typeface;

public class ControlWidget {
	
	private Texture mTexture;
	private Texture mTex;
	private TextureRegion mCircleTexture;
	
	private Sprite touchOne;
	private Sprite touchTwo;	
	
//	private final int spriteWidth = 64;
//	private final int spriteHeight = 64;
	private final int halfSpriteWidth = 32;
	private final int halfSpriteHeight = 32;
	
	private int lastX = 0;
	private int lastY = 0;
	
	private Perimiter perimiter;
	
	private int touches[] = {0,0,0,0};
	
	private Line edge;
	private Line centerLine;
	private Texture mFontTexture;
	private Font mFont;
	private ChangeableText angleText;
	
	private TextureRegion mParticleTextureRegion;
	
	// particle systems
	ParticleSystem particleSystem1a;
	ParticleSystem particleSystem1b;
	
	// Default color modifier goes from green to white
	ColorModifier particleColorModifierA = new ColorModifier(1f, 0f, 0f, 1f, 1f, 1f, 0f, 7f);
	ColorModifier particleColorModifierB = new ColorModifier(1f, 0f, 0f, 1f, 1f, 1f, 0f, 3f);
		
	private int cameraWidth;
	private int cameraHeight;
	
	private double angle = 0;
	
	public ControlWidget(Context context, final int camWidth, final int camHeight){
		
		cameraWidth = camWidth;
		cameraHeight = camHeight;
		
		this.mFontTexture = new Texture(256, 256, TextureOptions.BILINEAR_PREMULTIPLYALPHA);

		this.mFont = new Font(this.mFontTexture, Typeface.create(Typeface.DEFAULT, Typeface.BOLD), 24, true, Color.WHITE);

		// Load touch texture
		this.mTexture = new Texture(128, 128, TextureOptions.BILINEAR_PREMULTIPLYALPHA);
		this.mTex = new Texture(32, 32, TextureOptions.BILINEAR_PREMULTIPLYALPHA);
		
		this.mCircleTexture = TextureRegionFactory.createFromAsset(this.mTexture, context, "gfx/yellow_circle.png", 0, 0);

		this.mParticleTextureRegion = TextureRegionFactory.createFromAsset(this.mTex, context, "gfx/particle_point.png", 0, 0);

		
	}
	
	
	
	public void loadResources(Engine engine) {

		engine.getTextureManager().loadTexture(this.mTexture);
		engine.getTextureManager().loadTexture(this.mTex);
				
		engine.getTextureManager().loadTexture(this.mFontTexture);
		engine.getFontManager().loadFont(this.mFont);
	}
	
	
	public void loadScene(Scene scene) {
		// Touches
		edge = new Line(0,0,0,0);
		edge.setColor(1,1,1);
		scene.getTopLayer().addEntity(edge);
		
		centerLine = new Line(0,0,0,0);
		centerLine.setColor(1, 1, 1);
		scene.getTopLayer().addEntity(centerLine);
		perimiter = new Perimiter(64, scene);
		perimiter.setColor(1, 1, 1);

		/* Create the first node */
		touchOne = new Sprite(100, 100, this.mCircleTexture) {
				boolean mGrabbed = false;
				
				@Override
				public boolean onAreaTouched(final TouchEvent e, final float pTouchAreaLocalX, final float pTouchAreaLocalY) {
					switch(e.getAction()) {
						case TouchEvent.ACTION_DOWN:
							this.setScale(1.5f);
							
							this.mGrabbed = true;						
							break;
						case TouchEvent.ACTION_MOVE:
							if(this.mGrabbed) {
								this.setPosition(e.getX() - halfSpriteWidth, e.getY() - halfSpriteHeight);
								
								lastX = touches[0];
								lastY = touches[1];
								
								touches[0] = (int) e.getX();
								touches[1] = (int) e.getY();
								
								touches[2] = touches[2] - (lastX - touches[0]);
								touches[3] = touches[3] - (lastY - touches[1]);
								
								//update the position of the other node when you move the central node
								touchTwo.setPosition(touches[2] - halfSpriteWidth, touches[3] - halfSpriteHeight);

								perimiter.setPosition(e.getX(), e.getY());

								edge.setPosition(touches[0], touches[1],
										touches[2], touches[3]);
								
								centerLine.setPosition(touches[0], touches[1],
										touches[0], touches[1] + perimiter.radius);
								centerLine.setPosition(touches[0], touches[1],
										touches[0] + perimiter.radius,
										touches[1]);

								// update angle text disp
								angleText.setPosition(touches[0] + 40,
										touches[1] - 25);
								angleText.setText(Double.toString(perimiter.angle));


							}
							break;
						case TouchEvent.ACTION_UP:
							if(this.mGrabbed) {
								this.mGrabbed = false;
								this.setScale(1.0f);
							}
							break;
					}
					return true;
				}
			};
			
			/* Left to right Particle System. */
			
			{
		    particleSystem1a = new ParticleSystem(new PointParticleEmitter(0, cameraHeight), 20, 40, 200, this.mParticleTextureRegion);
			particleSystem1a.setBlendFunction(GL10.GL_SRC_ALPHA, GL10.GL_ONE);

			particleSystem1a.addParticleInitializer(new VelocityInitializer(15, 32, -60, -90));
			particleSystem1a.addParticleInitializer(new AccelerationInitializer(35, 65));

			particleSystem1a.addParticleModifier(new ScaleModifier(1.0f, 0.05f, 0, 3));
			particleSystem1a.addParticleModifier(new ExpireModifier(4.5f));
			particleSystem1a.addParticleModifier(new AlphaModifier(1.0f, 0.0f, 0f, 4.5f));
			particleSystem1a.addParticleModifier(particleColorModifierA);
			
		    particleSystem1b = new ParticleSystem(new PointParticleEmitter(0, cameraHeight), 10, 20, 100, this.mParticleTextureRegion);
			particleSystem1b.setBlendFunction(GL10.GL_SRC_ALPHA, GL10.GL_ONE);

			particleSystem1b.addParticleInitializer(new VelocityInitializer(30, 60, -90, -120));
			particleSystem1b.addParticleInitializer(new AccelerationInitializer(20, 40));

			particleSystem1b.addParticleModifier(new ScaleModifier(2.5f, 0.05f, 0, 7));
			particleSystem1b.addParticleModifier(new ExpireModifier(7f));
			particleSystem1b.addParticleModifier(new AlphaModifier(1.0f, 0.0f, 0f, 7.5f));
			particleSystem1b.addParticleModifier(particleColorModifierB);
			
			particleSystem1a.setParticlesSpawnEnabled(false);
			particleSystem1b.setParticlesSpawnEnabled(false);
			
			scene.getTopLayer().addEntity(particleSystem1a);
			scene.getTopLayer().addEntity(particleSystem1b);

		}

			
			
		scene.getTopLayer().addEntity(touchOne);
		scene.registerTouchArea(touchOne);
		
		// Create the second node //
		touchTwo = new Sprite(300, 100, this.mCircleTexture) {
			boolean mGrabbed = false;
			
			@Override
			public boolean onAreaTouched(final TouchEvent e, final float pTouchAreaLocalX, final float pTouchAreaLocalY) {
				switch(e.getAction()) {
					case TouchEvent.ACTION_DOWN:
						this.setScale(1.5f);
						this.mGrabbed = true;
						particleSystem1a.setParticlesSpawnEnabled(true);
						particleSystem1b.setParticlesSpawnEnabled(true);
						break;
					case TouchEvent.ACTION_MOVE:
						if(this.mGrabbed) {
							this.setPosition(e.getX() - halfSpriteWidth, e.getY() - halfSpriteHeight);
							touches[2] = (int) e.getX();
							touches[3] = (int) e.getY();

							perimiter.setPerimiterPoint(touches[2], touches[3]);

							edge.setPosition(touches[0], touches[1],
									touches[2], touches[3]);
							
							centerLine.setPosition(touches[0], touches[1],
									touches[0], touches[1] + perimiter.radius);

							angleText.setPosition(touches[0] + 40, touches[1] - 25);
							
							angleText.setText(Double.toString(perimiter.angle));
							
							BaseParticleEmitter emitterA = (BaseParticleEmitter) particleSystem1a.getParticleEmitter();
							emitterA.setCenter(touches[2], touches[3]);
							BaseParticleEmitter emitterB = (BaseParticleEmitter) particleSystem1b.getParticleEmitter();
							emitterB.setCenter(touches[2], touches[3]);

						}
						break;
					case TouchEvent.ACTION_UP:
						if(this.mGrabbed) {
							this.mGrabbed = false;
							this.setScale(1.0f);
							particleSystem1a.setParticlesSpawnEnabled(false);
							particleSystem1b.setParticlesSpawnEnabled(false);
						}
						break;
				}
				return true;
			}
		};
		
		scene.getTopLayer().addEntity(touchTwo);	
		scene.registerTouchArea(touchTwo);
		
		// Touches
		
		edge = new Line(0,0,0,0);
		edge.setColor(1,1,1);
		scene.getTopLayer().addEntity(edge);
		
		centerLine = new Line(0,0,0,0);
		centerLine.setColor(1, 1, 1);
		scene.getTopLayer().addEntity(centerLine);
	
		
		perimiter = new Perimiter(64, scene);
		perimiter.setColor(1, 1, 1);
		
		angleText = new ChangeableText(-100, -100, this.mFont, Double.toString(perimiter.getAngle()));
		scene.getTopLayer().addEntity(angleText);
	}
	
	
	public void setColor(final float r, final float g, final float b) {
		
		perimiter.setColor(r, g, b);
//		particleSystem1a.removeParticleInitializer(particleColorModifierA);
//		particleSystem1b.removeParticleInitializer(particleColorModifierB);
		particleColorModifierA = new ColorModifier(r,g,b,1f,1f,1f,0f,5f);
		particleColorModifierB = new ColorModifier(r,g,b,1f,1f,1f,0f,3f);
		
	}

}
  