package com.ghostmachine.android.twotouchxy;

import org.anddev.andengine.entity.primitive.Line;
import org.anddev.andengine.entity.scene.Scene;

import android.graphics.Point;
import android.util.Log;

public class Perimiter {
	Point one;
	Point two;
	
	float centerX;
	float centerY;
	
	double angle = 0;
	float radius;
	int segments;
	float step;
	
	// Colors
	float r;
	float g;
	float b;
	float a = 1f;
	
	Line[] lines;
	
	Line[] outlinesA;
	Line[] outlinesB;
	
	Scene scene;
   
	
	
	public Perimiter(int segments, Scene scene) {

		one = new Point();
		two = new Point();
		
		centerX = (one.x + two.x) / 2;
		centerY = (one.y + two.y) / 2;
		
		float tx = one.x - centerX;
		float ty = one.y - centerY;
		radius = (float) Math.sqrt((tx * tx) + (ty * ty));
		
		this.segments = segments;		
		lines = new Line[segments];
		outlinesA = new Line[segments / 2];
		outlinesB = new Line[segments / 2];
		
		this.scene = scene;
		
		step = (float) ((2 * Math.PI) / segments);
		
		init();
		
		draw();
	}
	
	private void init() {
		int j = 0;
		
		for (int i = 0; i < segments; i++) {
			lines[i] = new Line(0,0,0,0);
			lines[i].setLineWidth(6);
			lines[i].setColor(1, 1, 1);
			scene.getTopLayer().addEntity(lines[i]);
			
			if (i % 2 == 0) {
				outlinesA[j] = new Line(0,0,0,0);
				outlinesA[j].setLineWidth(3);
				outlinesA[j].setColor(0.75f, 0.75f, 0.75f);
				scene.getTopLayer().addEntity(outlinesA[j]);
				
				outlinesB[j] = new Line(0,0,0,0);
				outlinesB[j].setLineWidth(2);
				outlinesB[j].setColor(0.5f, 0.5f, 0.5f);
				scene.getTopLayer().addEntity(outlinesB[j]);
				j++;
			}
		}
	
	}
	
	public void setPoints(int x1, int y1, int x2, int y2) {
		one.set(x1, y1);
		two.set(x2, y2);
		centerX = (x1 + x2) / 2;
		centerY = (y1 + y2) / 2;
		
		float tx = one.x - centerX;
		float ty = one.y - centerY;
		
		radius = (float) Math.sqrt((tx * tx) + (ty * ty)) * 2;
		calculateAngle();
		draw();
	}

	public void draw() {	
//		Log.d("Perim Draw", Float.toString(centerX) + "  " + Float.toString(centerY) + "  " + Float.toString(radius));
		int j = 0;
		int k = 0;
		for (int i = 0; i < segments; i++) {
			Line l = lines[i];
			float x1 = (float) (radius * Math.cos(i * step));
			float y1 = (float) (radius * Math.sin(i * step));
			float x2 = (float) (radius * Math.cos((i + 1) * step));
			float y2 = (float) (radius * Math.sin((i + 1) * step));
			l.setPosition(x1 + centerX, y1 + centerY, x2 + centerX, y2 + centerY);
			
			// Wedge fill lines
			
			
			
			
			// Segmented lines
			if (i % 2 == 0) {
				Line la = outlinesA[j];
				float xa1 = (float) ((radius * 1.1) * Math.cos(i * step));
				float ya1 = (float) ((radius * 1.1) * Math.sin(i * step));
				float xa2 = (float) ((radius * 1.1) * Math.cos((i + 1) * step));
				float ya2 = (float) ((radius * 1.1) * Math.sin((i + 1) * step));
				la.setPosition(xa1 + centerX, ya1 + centerY, xa2 + centerX, ya2 + centerY);
				j++;
			} else {
				Line lb = outlinesB[k];
				float xb1 = (float) ((radius * 1.2) * Math.cos(i * step));
				float yb1 = (float) ((radius * 1.2) * Math.sin(i * step));
				float xb2 = (float) ((radius * 1.2) * Math.cos((i + 1) * step));
				float yb2 = (float) ((radius * 1.2) * Math.sin((i + 1) * step));
				lb.setPosition(xb1 + centerX, yb1 + centerY, xb2 + centerX, yb2 + centerY);
				k++;
			}
		}	
	}
	
	
	public void setColor(float r, float g, float b) {
		this.r = r;
		this.g = g;
		this.b = b;
		for (Line l : lines) {
			l.setColor(r, g, b);
		}
	}

	public void setPosition(float x, float y) {
		one.x = (int) x;
		one.y = (int) y;
		centerX = x;
		centerY = y;
		draw();
	}
	
	private float[] normalize(final float[] points) {
		// calculate distance
		final float dist = (float) Math.sqrt((points[0] * points[0]) + (points[1] * points[1]));
		// normalize
		final float normalX = points[0] / dist;
		final float normalY = points[1] / dist;
		
		return new float[] {normalX, normalY};
	}
	
	
	
	public double getAngle() {
		return angle;
	}

	private void calculateAngle() {
		int tx = one.x - two.x;
		int ty = one.y - two.y;
		final float[] normalizedValues = normalize(new float[] {tx, ty});
		// Calculate the angle
		angle = Math.atan2(normalizedValues[0], normalizedValues[1]);
		
		// map the values from -PI to PI, to 0 - 360;
		angle = Tools.map(angle, - (Math.PI), Math.PI, 360, 0);
	}

	public void setPerimiterPoint(int x, int y) {
		// TODO Auto-generated method stub
		two.x = x;
		two.y = y;
		radius = calculateRadius();
		//update angle value
		calculateAngle();
		draw();
	}

	private float calculateRadius() {
		// TODO Auto-generated method stub
		int tx = Math.abs(two.x - one.x);
		int ty = Math.abs(two.y - one.y);
		return (float) Math.sqrt((tx * tx) + (ty * ty));
	}
}
