package com.ghostmachine.android.twotouchxy;

import java.util.Stack;

import org.anddev.andengine.engine.Engine;
import org.anddev.andengine.engine.camera.Camera;
import org.anddev.andengine.engine.options.EngineOptions;
import org.anddev.andengine.engine.options.EngineOptions.ScreenOrientation;
import org.anddev.andengine.engine.options.resolutionpolicy.FillResolutionPolicy;
import org.anddev.andengine.entity.scene.Scene;
import org.anddev.andengine.entity.scene.background.ColorBackground;
import org.anddev.andengine.entity.util.FPSLogger;
import org.anddev.andengine.extension.input.touch.controller.MultiTouch;
import org.anddev.andengine.extension.input.touch.controller.MultiTouchController;
import org.anddev.andengine.extension.input.touch.controller.MultiTouchException;
import org.anddev.andengine.sensor.accelerometer.AccelerometerData;
import org.anddev.andengine.sensor.accelerometer.IAccelerometerListener;

import android.graphics.Color;
import android.view.Display;
import android.widget.Toast;

/**
 * @author Nicolas Gramlich
 * @since 18:20:40 - 18.06.2010
 */
public class TwoTouchXY extends BaseExample implements IAccelerometerListener{
	// ===========================================================
	// Constants
	// ===========================================================

	private static final int CAMERA_WIDTH = 1024;
	private static final int CAMERA_HEIGHT = 600;

	// ===========================================================
	// Fields
	// ===========================================================

	private Camera mCamera;

	int height;
	int width;
	
	private ControlWidget controlOne;
	private ControlWidget controlTwo;
	private ControlWidget controlThree;
	
	private Stack colors = new Stack();

	// ===========================================================
	// Constructors
	// ===========================================================

	// ===========================================================
	// Getter & Setter
	// ===========================================================

	// ===========================================================
	// Methods for/from SuperClass/Interfaces
	// ===========================================================

	@Override
	public Engine onLoadEngine() {
		Display display = getWindowManager().getDefaultDisplay();
		width = display.getWidth();
		height = display.getHeight();

		this.mCamera = new Camera(0, 0, width, height);
		final Engine engine = new Engine(new EngineOptions(true, ScreenOrientation.PORTRAIT, new FillResolutionPolicy(), this.mCamera));

		try {
			if(MultiTouch.isSupported(this)) {
				engine.setTouchController(new MultiTouchController());
				if(MultiTouch.isSupportedDistinct(this)) {
					Toast.makeText(this, "MultiTouch detected --> Drag multiple Sprites with multiple fingers!", Toast.LENGTH_LONG).show();
				} else {
					Toast.makeText(this, "MultiTouch detected --> Drag multiple Sprites with multiple fingers!\n\n(Your device might have problems to distinguish between separate fingers.)", Toast.LENGTH_LONG).show();
				}
			} else {
				Toast.makeText(this, "Sorry your device does NOT support MultiTouch!\n\n(Falling back to SingleTouch.)", Toast.LENGTH_LONG).show();
			}
		} catch (final MultiTouchException e) {
			Toast.makeText(this, "Sorry your Android Version does NOT support MultiTouch!\n\n(Falling back to SingleTouch.)", Toast.LENGTH_LONG).show();
		}
		
		// Add the control widget
		controlOne = new ControlWidget(this, width, height);
		controlTwo = new ControlWidget(this, width, height);
//		controlThree = new ControlWidget(this, width, height);
		
		// Color stack
		colors.add(Color.BLUE);
		colors.add(Color.GREEN);
		colors.add(Color.RED);
		colors.add(Color.WHITE);
		colors.add(Color.YELLOW);
		
		
		
		return engine;
	}

	@Override
	public void onLoadResources() {
		
		controlOne.loadResources(mEngine);
		controlTwo.loadResources(mEngine);
//		controlThree.loadResources(mEngine);
		
		this.enableAccelerometerSensor(this);
	}

	@Override
	public Scene onLoadScene() {
		this.mEngine.registerUpdateHandler(new FPSLogger());
		
		final Scene scene = new Scene(1);
		scene.setOnAreaTouchTraversalFrontToBack();
		scene.setBackground(new ColorBackground(0.0f, 0.0f, 0.0f));
		
		controlOne.loadScene(scene);
		controlTwo.loadScene(scene);
//		controlThree.loadScene(scene);
		
		scene.setTouchAreaBindingEnabled(true);

		return scene;
	}


	@Override
	public void onLoadComplete() {

	}

	@Override
	public void onAccelerometerChanged(AccelerometerData ad) {
//		Log.d("Accelerometer", Double.toString(ad.getY()));
		
	}
	

}








